/************************************************************************************************

 Author        : ADI - Apps            www.analog.com

 Date          : July 2009

 File          : I2C_Master.c

 Hardware      : ADuC7122

 Description   : This I2C Master example shows how initialise the I2C peripheral in Master mode
 				This simple example transmits 6 bytes to Slave address 0xA0 after a reset.
				When an XIRQ0 interrupt occurs, the Master will then read back 6 bytes from the Slave.		
*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT19 0x80000
#define BIT20 0x100000
#define I2C_RD_BIT 0x1

#include <ADuC7122.h>
# include "stdio.h"
# include "string.h"

unsigned char szTxData[] = {0x1, 0x2, 0x10, 0x20, 0x40, 0x80};	// Array to send to Slave
unsigned char ucTxCount = 0;  			// Array index variable for szTxData[]
unsigned char szRxData[6];				// Array for reading Data from Slave
unsigned char ucRxCount = 0;  			// Array index variable for szRxData[]
unsigned char ucWaitingForXIRQ0 = 1; 	// Flag to begin reading from Slave - controlled by XIRQ0
int main(void)
{
    POWKEY1 = 0x01;		         		// Configure CPU Clock for 41.78MHz, CD=0
 	POWCON = 0x00;
 	POWKEY2 = 0xF4; 

    // Configure P0.0 and P0.1 for I2C mode
	GP0CON = BIT0 + BIT4; 				// Select I2C alternative function for P0.0 & P0.1

	// Enable I2C Master mode, baud rate and interrupt sources
	I2C0MCTL = BIT0 + BIT4  			// Enable I2C Master + Enable Rx interrupt
			+ BIT5 + BIT8;  			// Enable Tx interrupt + Enable transmission complete interrupt
	I2C0DIV  = 0x3131;	   				// Select 100.3kHz clock rate
	IRQEN = BIT15 + BIT19; 				// Enable I2C Master and XIRQ0 interrupts
        IRQCONE=BIT0+BIT1;          	// External IRQ0 triggers on falling edge
 
	// Begin Master Transmit sequence
	ucTxCount = 0;
	I2C0FSTA = BIT9;					// Flush Master Tx FIFO
	I2C0FSTA &= ~BIT9;
	I2C0MTX = szTxData[ucTxCount++];
	I2C0ADR0 = 0xA0; 				    // Write to the slave
	ucWaitingForXIRQ0 = 1;
	ucRxCount = 0;
	while (1)
	{
	 	if (ucWaitingForXIRQ0 == 0)		// Wait for XIRQ0 to trigger - cleared in XIRQ0 ISR
		{
			ucWaitingForXIRQ0 = 1;
			ucRxCount = 0;
			I2C0MCNT0 = 5; 				// Read back 6 bytes
			I2C0ADR0 =  0xA1;			// Begin Read sequence from Slave
		}
	}
}

void IRQ_Handler(void)  __irq 
 {
	unsigned long IRQSTATUS = 0;
	unsigned int I2C0MSTATUS = 0;

	IRQSTATUS = IRQSTA;	   						// Read off IRQSTA register
	if ((IRQSTATUS & BIT19) == BIT19)			//XIRQ0 interrupt source
	{
		ucWaitingForXIRQ0 = 0;					// Enable Read request from Slave.
	        IRQCLR = BIT19;
        }

	if ((IRQSTATUS & BIT15) == BIT15)			//If I2C Master interrupt source
	{										
	   I2C0MSTATUS = I2C0MSTA;
	   if ((I2C0MSTATUS & BIT2) == BIT2)		// If I2C Master Tx IRQ
	   {
			if (ucTxCount < 6)					// Have max 6 bytes been sent?
				I2C0MTX = szTxData[ucTxCount++];// Load Tx buffer
	   }
	   if ((I2C0MSTATUS & BIT3) == BIT3) 		// If I2C Master Rx IRQ
	   {
			if (ucRxCount < 6)					// Have max 6 bytes been received?
			{
				szRxData[ucRxCount] = I2C0MRX;  // Read Rx buffer
				ucRxCount++;
			}
	   }
	}
}


